#include "stdio.h"
#include "osbind.h"
/* 
*****************************************************************************
*
*      LOOK0.PRG  Copyright 1987 By Dan Rhea and Robert Birmingham 
*                      May be copied but not sold
*
* This program displays 32K of memory as graphics data. This is best done on
* a monochrome screen since it only has one bit plane so the data in the CRT
* is a true picture of low memory. Medium and Low-Rez look intresting but
* do not show a true image of memory since they are multi-plane images.
*
* Commands: Down arrow:  Move down in memory 32K bytes
*           Up arrow:    Move up in memory 32K bytes
*           Left arrow:  Move down in memory 1280 bytes
*           Right arrow: Move up in memory 1280 bytes
*           Help:        Show current screen address
*           Undo:        Return to desktop or shell
*
* Other info: This program will accept addresses up to 16MB - 32K so if it is
*             run on a 520ST, 1040ST or a Mega, the screen will repeat every
*             4MB. On machines with less than 4MB you will have screens that
*             display what appears to be static, this is actually non-present
*             memory. 
*
* History: 08/18/87 DR & RB Completed the initial version that displayed
*                   location 0-32K.
*          08/27/87 DR Added intro and exit alerts.
*          09/01/87 DR Added new alerts, made the old ones nicer and added the
*                   facility to "Page" and "Move" up and down in memory.
*          09/02/87 DR Added info alerts and the current address alert.
*
******************************************************************************
**
** Defines
*/

#define UNDO    0x00610000L     /* Undo key */
#define HELP    0x00620000L     /* Help key */
#define PGUP    0x00480000L     /* Up arrow key */
#define PGDN    0x00500000L     /* Down arrow key */
#define MVUP    0x004D0000L     /* Right arrow key */
#define MVDN    0x004B0000L     /* Left arrow key */
#define MOVE    1280L		/* One scrool worth */
#define PAGE    32000L          /* One screen worth */
#define MAXI    0x00FA0000L     /* 16MB */

/*
** Workstation variables
*/

int  contrl[20], intin[12], ptsin[128], intout[128], ptsout[128],
     work_in[11], work_out[57], handle, dummy, i;

/*
** Let's get this silly thing started
*/

main () {

   /*
   ** Declare local variables and alert boxes
   */

   long int igot,                       /* Input character */
             mema,                      /* Pointer to low mem */
             norm;                      /* Real screen address */

   int button;                          /* alert button */

   static char curra[191];              /* current addr alert */

   static char intro[] = {
"[0]\
[      LOOK0  Ver. 1.04 \
|       Copyright 1987 \
|             by \
|Dan Rhea & Robert Birmingham \
| May be copied but not sold ]\
[Go for it!|Help]"                      /* Intro alert */
   };

   static char howt0[] = {
"[0]\
[Undo=Quit  Help=Address \
|  Page up 32K bytes \
|  Page down 32K bytes \
|  Move up 1280 bytes \
|  Move down 1280 bytes ]\
[More|Enough!]"                         /* Help 1 alert */
   };

   static char howt1[] = {
"[0]\
[If you have a 520ST, 1040ST, \
|or a MegaST the display will \
|repeat every 4 Megs. This is \
|normal. Static on the screen \
|is non-present memory. ]\
[zzzzzzzz...]"                          /* Help 2 alert */
   };

   static char leave[] = {
"[0]\
[ \
|    Complements of \
|Dade Atari Users Group \
|    Miami, Florida \
| ]\
[D..U.G]"                              /* Goodbye Alert */
   };

   /*
   ** Like here we go dudes (and dudetts), fersure
   */

   mema = 0L;                           /* Set initial address */
   appl_init ();                        /* Init appl */

   /*
   ** Open workstation, define the mouse
   */

   handle = graf_handle (&dummy, &dummy, &dummy, &dummy);
   for (i=0; i < 10; work_in[i++] = 1); /* Get parameters */
   work_in[10] = 2;                     /* Set up */
   v_opnvwk(work_in,&handle,work_out);  /* Open workstation */
   dummy = graf_mouse(3,0x0L);          /* Define mouse */

   /*
   ** Introduce ourselves
   */

   v_show_c(handle);                    /* Show the mouse */
   button = form_alert(1,intro);        /* Intro program */

   /*
   ** Program info
   */

   if (button == 2) {                   /* Need help? */
      button = form_alert(1,howt0);     /* Give help */
      if (button == 1)                  /* Need more info? */
         button = form_alert(1,howt1);  /* Give more help */
   }

   /* 
   ** Save current screen address, set it to the start of the
   ** system memory (loc 0), and wait for an UNDO to exit, an
   ** UpArrow for next 32K, a DownArrow for previous 32K or a
   ** Help for the current screen address. Also RightArrow for
   ** the next 1280 bytes and LeftArrow for the previous 1280
   ** bytes
   */

   igot = 0;                            /* Init igot */
   v_hide_c(handle);                    /* Hide mouse */
   norm = (long int)Physbase();         /* Save normal screen address */ 
   Setscreen (mema,mema,-1);            /* Make location 0 the screen */
   while (igot != UNDO) {               /* Exit on UNDO */

      /*
      ** Watch for user input, if any, grab it
      */

      if (Cconis() != 0) {              /* Any input? */
         igot = Crawcin();		/* Get input */

         /*
         ** Process up arrow
         */

         if (igot == PGUP) {            /* Shift up arrow? */
            if (mema < (MAXI-PAGE))     /* Top of possible memory? */
               mema=mema+PAGE;          /* Move up in memory */
            Setscreen(mema,mema,-1);    /* Show new screen */

         /* 
         ** Process down arrow (Not valid if address is < 32K)
         */

         } else if (igot == PGDN) {     /* Page down 32K bytes */
            if (mema >= PAGE)           /* Anywhere to go? */
               mema=mema-PAGE;          /* Move down in memory */
            Setscreen(mema,mema,-1);    /* Show new screen */

         /*
         ** Process right arrow
         */

         } else if (igot == MVUP) {     /* Move up 1280 bytes */
            if (mema < (MAXI-PAGE))     /* Top of possible memory? */
               mema=mema+MOVE;          /* Move up in memory */
            Setscreen(mema,mema,-1);    /* Show new screen */

         /*
         ** Process left arrow (Not valid if addres is < 1280 bytes)
         */

         } else if (igot == MVDN) {     /* Move down 1280 bytes */
            if (mema >= MOVE)           /* Anywhere to go? */
               mema=mema-MOVE;          /* Move down in memory */
            Setscreen(mema,mema,-1);	/* Show new screen */

         /*
         ** Process Help (current address)
         */

         } else if (igot == HELP) {     /* Help? */
            sprintf(curra,"[0][Current |memory |address: |%ld |0x%lx ][Ok]\0",
            mema,mema);                 /* Set variable alert box */
            Setscreen(norm,norm,-1);    /* Normal screen */
            v_show_c(handle);           /* Show the mouse */
            button=form_alert(1,curra); /* Show address */
            v_hide_c(handle);           /* Hide the mouse */
            Setscreen(mema,mema,-1);    /* Back to viewing memory */
         }
      }
   }
   Setscreen (norm,norm,-1);            /* Restore screen to normal */

   /*
   ** Say goodbye
   */

   v_show_c(handle);                    /* Show the mouse */
   button = form_alert(1,leave);        /* Say goodbye */
   v_hide_c(handle);                    /* Hide it again */
   v_clsvwk(handle);                    /* Close the screen */
   appl_exit ();                        /* quit */

} /* End of LOOK0 */
